//	Desktop.java : this is a Java source code file for the program Facade.
//	Copyright 1998, Andrew S. Downs
//	andrew.downs@tulane.edu
//
//	This source code is distributed as freeware.
//	Just keep this author information in the file.  Enjoy!

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.net.*;
import java.util.*;
import com.sun.java.swing.*;

public class Desktop extends Window implements Serializable, MouseListener, MouseMotionListener, TimerEventListener {
	transient boolean bSplash = false, firstRepaint = true, dragInProgress = false;

	transient int delay;
	transient int menuBarHeight;
	transient int res;
	transient int IMG_SPLASH = 0;
	transient int IMG_APPLE = 1;
	transient int IMG_TRASH = 2;
	transient int IMG_TRASH_FULL = 3;
	transient int IMG_TRASH_MASK = 8;
	transient int IMG_TRASH_FULL_MASK = 11;
	transient int IMG_DRIVE_MASK = 9;
	transient int IMG_TRASH_STATIC = 10;
	transient int IMG_DRIVE = 4;
	transient int IMG_DOC = 5;
	transient int IMG_FOLDER = 6;
	transient int IMG_UP_ARROW = 7;
	transient int activeMenu = -1, activeMenuItem = -1;
	transient int initialX = 0;
	transient int initialY = 0;

	transient Dimension d;

	transient DesktopMenuBar dmb;
	transient DesktopImage imgSplash, imgFolder, imgDriveMask, imgTrashStatic, imgTrash, imgTrashMask, imgTrashFull, imgTrashFullMask, imgDrive, imgApple;//, imgTrash, imgTrashFull, imgDrive, imgDoc, imgFolder;
	transient DesktopImage dragItem = new DesktopImage();
	transient DesktopImage highlightedImage = new DesktopImage();
	
	transient Font menuFont, iconFont;
	
	transient Rectangle activeMenuRect = new Rectangle( -1, -1, 0, 0 );
	transient Rectangle activeMenuItemRect = new Rectangle( -1, -1, 0, 0 );
	transient static Rectangle tempClipRect = new Rectangle( -1, -1, 0, 0 );
	
	transient String fileSep, pathSep, lineSep;

	transient Toolkit tk = Toolkit.getDefaultToolkit();

	Vector vectorImages = new Vector();
	static Vector vectorIcons = new Vector();
	transient Vector vectorMenus = new Vector();
	static Vector vectorWindows = new Vector();
	Vector vectorJunk = new Vector();
	public static Vector vectorUtil = new Vector();	
	transient Vector trash = new Vector();

	Image offscreenImage;
	Graphics g;

	Desktop() {
		// General setup.
		super( new Frame() );
		this.setLayout( null );
		this.setBackground( Color.lightGray );
		this.menuFont = new Font( "Dialog", Font.BOLD, 12 );
		this.iconFont = new Font( "SansSerif", Font.PLAIN, 12 );
		this.setFont( this.menuFont );

	DesktopImage imgDoc;

		// Initialize instance variables.
		this.fileSep = Global.fileSep;
		this.bSplash = true;
		imgSplash = new DesktopImage();
		imgApple = new DesktopImage();
		imgTrash = new DesktopImage();
		imgDrive = new DesktopImage();
		imgTrashFull = new DesktopImage();
		imgDoc = new DesktopImage();
		imgFolder = new DesktopImage();
		imgDriveMask = new DesktopImage();
		imgTrashMask = new DesktopImage();
		imgTrashFullMask = new DesktopImage();
		
		MediaTracker mt = new MediaTracker( this );
		URL u;

		// Get splash image.
		try {
    		u = this.getClass().getResource( Global.splashString );
			imgSplash.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgSplash.getImage(), IMG_SPLASH );

			mt.waitForID( IMG_SPLASH );

			imgSplash.setX( this.getX() + ( ( this.getWidth() / 2 ) - ( imgSplash.getImage().getWidth( this ) / 2 ) ) );
			imgSplash.setY( this.getY() + ( ( this.getHeight() / 2 ) - ( imgSplash.getImage().getHeight( this ) / 2 ) ) );
		}
		catch ( MalformedURLException e ) {
			System.out.println( "MalformedURLException" );
		}
		catch ( IOException e ) {
			System.out.println( "IOException" );
		}
		catch ( InterruptedException e ) {
			System.out.println( "InterruptedException" );
		}

		// Set Window properties based on screen size.
		res = tk.getScreenResolution();
		d = tk.getScreenSize();
		this.setBounds( Global.defaultX, Global.defaultY, d.width, d.height );
		this.setMenuBarHeight( Global.defaultMenuBarHeight );
		this.setVisible( true );

		offscreenImage = createImage( this.getBounds().width, this.getBounds().height );
		g = offscreenImage.getGraphics();

		// Display splash screen.
		this.repaint();
		mt.removeImage( imgSplash.getImage(), IMG_SPLASH );

		// Get all images.
		try {
    		u = this.getClass().getResource( Global.appleString );
			imgApple.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgApple.getImage(), IMG_APPLE );

     		u = this.getClass().getResource( Global.trashString );
			imgTrash.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgTrash.getImage(), IMG_TRASH );

   			u = this.getClass().getResource( Global.trashFullString );
			imgTrashFull.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgTrashFull.getImage(), IMG_TRASH_FULL );

   			u = this.getClass().getResource( Global.driveString );
			imgDrive.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgDrive.getImage(), IMG_DRIVE );

   			u = this.getClass().getResource( Global.docString );
			imgDoc.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgDoc.getImage(), IMG_DOC );

   			u = this.getClass().getResource( Global.folderString );
			imgFolder.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgFolder.getImage(), IMG_FOLDER );

   			u = this.getClass().getResource( Global.trashMaskString );
			imgTrashMask.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgTrashMask.getImage(), IMG_TRASH_MASK );

   			u = this.getClass().getResource( Global.trashFullMaskString );
			imgTrashFullMask.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgTrashFullMask.getImage(), IMG_TRASH_FULL_MASK );

   			u = this.getClass().getResource( Global.driveMaskString );
			imgDriveMask.setImage( this.createImage( ( ImageProducer )( u.getContent() ) ) );
			mt.addImage( imgDriveMask.getImage(), IMG_DRIVE_MASK );

//System.out.println( "Waiting..." );
			mt.waitForAll();
//System.out.println( "Finished waiting." );

			imgApple.setX( this.getX() + ( 2 * Global.defaultMenuHSep ) );
			imgApple.setY( this.getY() + Global.defaultMenuItemVSep );

			imgTrash.setX( this.getX() + this.getWidth() - ( imgTrash.getImage().getWidth( this ) * 2 ) );
			imgTrash.setY( this.getY() + this.getHeight() - ( imgTrash.getImage().getHeight( this ) * 3 ) );
			imgTrash.setLabel( Global.trashDisplayString );
			imgTrash.setLabelLocation( imgTrash.getX() + ( imgTrash.getImage().getWidth( this ) / 2 ) - ( this.getGraphics().getFontMetrics().stringWidth( imgTrash.getLabel() ) / 2 ), imgTrash.getY() + ( imgTrash.getImage().getHeight( this ) ) + ( this.getGraphics().getFontMetrics().getHeight() ) );

			imgTrashFull.setX( imgTrash.getX() );
			imgTrashFull.setY( imgTrash.getY() );
			imgTrashFull.setLabel( Global.trashDisplayString );
			imgTrashFull.setLabelLocation( imgTrash.getX() + ( imgTrash.getImage().getWidth( this ) / 2 ) - ( this.getGraphics().getFontMetrics().stringWidth( imgTrash.getLabel() ) / 2 ), imgTrash.getY() + ( imgTrash.getImage().getHeight( this ) ) + ( this.getGraphics().getFontMetrics().getHeight() ) );
			
			imgDrive.setX( this.getX() + this.getWidth() - ( imgDrive.getImage().getWidth( this ) * 2 ) );
			imgDrive.setY( this.getY() + this.menuBarHeight + imgDrive.getImage().getHeight( this ) );

			String defaultVolume = System.getProperty( "user.home" );
			
			int loc = 0;

			//pathSep sep02
			StringBuffer sb2 = new StringBuffer();
			sb2.append( defaultVolume.charAt( 0 ) );
			int len = defaultVolume.length();
			int m = 0;
			while ( ++m < len ) {
				sb2.append( defaultVolume.charAt( m ) );

				if ( ( defaultVolume.charAt( m ) == Global.sep02 ) || ( defaultVolume.charAt( m ) == Global.sep01 ) )
					break;
			}

//System.out.println( "Got this far..." );
/*
			// Parse root volume name.
			loc = defaultVolume.indexOf( Global.fileSep );
			
			// Volume name includes first '\'.
			// Rework this for MacOS.
			if ( loc > 0 )
{//			if ( ( loc > 0 ) && ( loc < defaultVolume.length() ) )
				defaultVolume = new String( defaultVolume.substring( 0, loc - 1 ) );
//				defaultVolume = defaultVolume.substring( 0, loc + 1 );
System.out.println( Global.fileSep );
}	
			
			// Parse root volume name.
			loc = defaultVolume.indexOf( Global.pathSep );

			// Volume name includes first '\'.
			// Rework this for MacOS.
			if ( loc > 0 )
{//			if ( ( loc > 0 ) && ( loc < defaultVolume.length() ) )
				defaultVolume = new String( defaultVolume.substring( 0, loc - 1 ) );
//				defaultVolume = defaultVolume.substring( 0, loc + 1 );
System.out.println( Global.pathSep );
}	
			// Parse root volume name.
			loc = defaultVolume.indexOf( Global.sep01 );
			
			// Volume name includes first '\'.
			// Rework this for MacOS.
			if ( loc > 0 )
{//			if ( ( loc > 0 ) && ( loc < defaultVolume.length() ) )
				defaultVolume = new String( defaultVolume.substring( 0, loc - 1 ) );
//				defaultVolume = defaultVolume.substring( 0, loc + 1 );
System.out.println( Global.sep01 );
}	
			
			// Parse root volume name.
			loc = defaultVolume.indexOf( Global.sep02 );
			
			// Volume name includes first '/'.
			// Rework this for MacOS.
			if ( loc > 0 )
{//			if ( ( loc > 0 ) && ( loc < defaultVolume.length() ) )
				defaultVolume = new String( defaultVolume.substring( 0, loc - 1 ) );
//				defaultVolume = defaultVolume.substring( 0, loc + 1 );
System.out.println( Global.sep02 );
}	
	
			// Parse root volume name.
			loc = defaultVolume.indexOf( Global.sep03 );
			
			// Volume name includes first '/'.
			// Rework this for MacOS.
			if ( loc > 0 )
{//			if ( ( loc > 0 ) && ( loc < defaultVolume.length() ) )
				defaultVolume = new String( defaultVolume.substring( 0, loc - 1 ) );
//				defaultVolume = defaultVolume.substring( 0, loc + 1 );
System.out.println( Global.sep03 );
}	
*/
			defaultVolume = sb2.toString();
			imgDrive.setPath( "\\" );
//			imgDrive.setPath( defaultVolume );
		
/*
			// Remove leading '\' character.
			loc = defaultVolume.indexOf( Global.sep01 );

			if ( loc == 0 ) {
				defaultVolume = new String( defaultVolume.substring( 1 ) );
			}
			
			// Remove leading '/' character.
			loc = defaultVolume.indexOf( Global.sep02 );

			if ( loc == 0 ) {
				defaultVolume = new String( defaultVolume.substring( 1 ) );
			}
*/
			sb2 = new StringBuffer();
			len = defaultVolume.length();
			m = 0;
			while ( m < len ) {
				if ( ( defaultVolume.charAt( m ) != Global.sep02 ) && ( defaultVolume.charAt( m ) != Global.sep01 ) )
					sb2.append( defaultVolume.charAt( m ) );

				m++;
			}
			
			defaultVolume = sb2.toString();
			
			// Volume name includes first '%20'.
			// Rework this for MacOS.
			loc = defaultVolume.indexOf( Global.spaceSep );

			while ( ( loc >= 0 ) && ( loc < defaultVolume.length() ) ) {
				String s1 = new String( defaultVolume.substring( 0, loc ) );
				String s2 = new String( defaultVolume.substring( loc + Global.spaceSep.length() ) );
				defaultVolume = new String( s1 + " " + s2 );
				loc = defaultVolume.indexOf( Global.spaceSep );
			}

			imgDrive.setLabel( defaultVolume );
			imgDrive.setLabelLocation( imgDrive.getX() + ( imgDrive.getImage().getWidth( this ) / 2 ) - ( this.getGraphics().getFontMetrics().stringWidth( imgDrive.getLabel() ) / 2 ), imgDrive.getY() + ( imgDrive.getImage().getHeight( this ) ) + ( this.getGraphics().getFontMetrics().getHeight() ) );

//System.out.println( "Calling doDelay()..." );
			
//			this.doDelay();
//System.out.println( "Finished calling doDelay()..." );
		}
		catch ( MalformedURLException e ) {
			System.out.println( "MalformedURLException" );
		}
		catch ( IOException e ) {
			System.out.println( "IOException" );
		}
		catch ( InterruptedException e ) {
			System.out.println( "InterruptedException" );
		}

		vectorImages.addElement( imgTrash );
		vectorImages.addElement( imgDrive );
//		vectorImages.addElement( imgFolder );
//		vectorImages.addElement( imgUpArrow );

		vectorUtil.addElement( imgDoc );
		vectorUtil.addElement( imgFolder );
		vectorUtil.addElement( imgTrashMask );
		vectorUtil.addElement( imgDriveMask );
		vectorUtil.addElement( imgTrashFull );
		vectorUtil.addElement( imgTrash );
		vectorUtil.addElement( imgTrashFullMask );

		DesktopFolder.setImage( imgFolder.getImage() );
		DesktopDoc.setImage( imgDoc.getImage() );

//System.out.println( "Checking the trash..." );
		// Initialize the trash.
		if ( !this.checkTrashExists() )
			this.createTrashDir();
		
		// If the trash is full, set the proper icon.
		DesktopImage di = ( ( DesktopImage )this.vectorImages.elementAt( 0 ) );
		String path = di.getPath();
		File f = new File( path );
		if ( f.exists() ) {
			String array[] = f.list();
				
			if ( ( array != null ) && ( array.length > 0 ) )
				this.vectorImages.setElementAt( this.imgTrashFull, 0 );
		}

		this.imgTrashFull.setPath( di.getPath() );
//System.out.println( "About to restore state." );

		this.restoreState();

		this.setupMenuBar();

		// Setup event listeners.
		this.addMouseListener( ( MouseListener )this );
		this.addMouseMotionListener( ( MouseMotionListener )this );
		
		this.bSplash = false;
		this.repaint();
		//this.redrawOpenFrames();
	}

	public void setupMenuBar() {
		Graphics g = this.getGraphics();
		FontMetrics theFontMetrics = g.getFontMetrics();
		
		dmb = new DesktopMenuBar();
		
// Changed.
//		dmb.setParentObj( this );

//		dmb = new DesktopMenuBar( this.getGraphics() );
//		this.add( dmb );
		dmb.setBounds( this.getBounds().x, this.getBounds().y, this.getBounds().width, Global.defaultMenuBarHeight );
//		dmb.setBounds( this.getBounds().x, this.getBounds().y, this.getBounds().width, this.getBounds().height );
		dmb.setBackground( Color.white );
		dmb.setForeground( Color.black );
		
// Changed.
//		dmb.setGraphics( this.getGraphics() );
		
		// Create menus and their menu items.
		DesktopImageMenu dim;
		Vector vectorMenuItems;
		DesktopMenuItem dmi;

		int newY = this.getY() + this.getMenuBarHeight() - ( this.getMenuBarHeight() / 3 );
		
		int newX = this.getX();
//		newX += Global.defaultMenuHSep;

		// Create Apple menu.
		dim = new DesktopImageMenu();
		dim.setImage( imgApple.getImage() );
		dim.setX( imgApple.getX() );
		dim.setY( imgApple.getY() );
		dmi = new DesktopMenuItem( Global.menuItemAboutMac );
		dmi.setEnabled( false );
		dim.getVector().addElement( dmi );
		dim.setItemLocations( this.getGraphics().getFontMetrics() );
		dmb.getVector().addElement( dim );
		
		newX = imgApple.getX();
		newX += ( 2 * Global.defaultMenuHSep );
		newX += ( ( DesktopImageMenu )dmb.getVector().elementAt( 0 ) ).getImage().getWidth( this );

		DesktopMenu dm;

		// Create File menu.
		dm = new DesktopMenu();
		dm.setLabel( Global.menuFile );
		dm.setX( newX );
		dm.setY( newY );
		dmi = new DesktopMenuItem( Global.menuItemNew );
		dmi.setEnabled( true );
		dm.getVector().addElement( dmi );
		dmi = new DesktopMenuItem( Global.menuItemOpen );
		dmi.setEnabled( false );
		dm.getVector().addElement( dmi );
		dmi = new DesktopMenuItem( Global.menuItemClose );
		dmi.setEnabled( false );
		dm.getVector().addElement( dmi );

		dm.setItemLocations( this.getGraphics().getFontMetrics() );
		dmb.getVector().addElement( dm );

		newX += ( 2 * Global.defaultMenuHSep );
		newX += theFontMetrics.stringWidth( dm.getLabel() );

		// Create Edit menu.
		dm = new DesktopMenu();
		dm.setLabel( Global.menuEdit );
		dm.setX( newX );
		dm.setY( newY );
		dmi = new DesktopMenuItem( Global.menuItemCut );
		dmi.setEnabled( false );
		dm.getVector().addElement( dmi );
		dmi = new DesktopMenuItem( Global.menuItemCopy );
		dmi.setEnabled( false );
		dm.getVector().addElement( dmi );
		dmi = new DesktopMenuItem( Global.menuItemPaste );
		dmi.setEnabled( false );
		dm.getVector().addElement( dmi );

		dm.setItemLocations( this.getGraphics().getFontMetrics() );
		dmb.getVector().addElement( dm );

		newX += ( 2 * Global.defaultMenuHSep );
		newX += theFontMetrics.stringWidth( dm.getLabel() );

		// Create Special menu.
		dm = new DesktopMenu();
		dm.setLabel( Global.menuSpecial );
		dm.setX( newX );
		dm.setY( newY );
		dmi = new DesktopMenuItem( Global.menuItemEmptyTrash );
		dmi.setEnabled( false );
		dm.getVector().addElement( dmi );
		dmi = new DesktopMenuItem( Global.menuSep );
		dmi.setEnabled( false );
		dm.getVector().addElement( dmi );
		dmi = new DesktopMenuItem( Global.menuItemRestart );
		dmi.setEnabled( true );
		dm.getVector().addElement( dmi );
		dmi = new DesktopMenuItem( Global.menuItemShutdown );
		dmi.setEnabled( true );
		dm.getVector().addElement( dmi );

		dm.setItemLocations( this.getGraphics().getFontMetrics() );
		dmb.getVector().addElement( dm );
	
	}
	
	public boolean unhighlightMenuItem( MouseEvent e ) {
		boolean eventHandled = false;
		Graphics g = this.getGraphics();
		FontMetrics theFontMetrics = g.getFontMetrics();

		// Handle unhighlighting.
		if ( this.activeMenu != -1 ) {
			// Get menu object.
			int i = this.activeMenu;
			DesktopMenu d = ( DesktopMenu )dmb.getVector().elementAt( i );
			
			// Get menu item vector.
			Vector v = d.getVector();

			// Unhighlight current menu item.
			// If Point is outside menu item, redraw the menu item.
			if ( this.activeMenuItem != -1 )  {
				if ( !activeMenuItemRect.contains( e.getX(), e.getY() ) ) {
					// Redraw menu background behind this menu item.
					g.setColor( dmb.getBackground() );
					g.fillRect( activeMenuItemRect.x, activeMenuItemRect.y, activeMenuItemRect.width, activeMenuItemRect.height );

					// Redraw menu item.
					g.setColor( Color.black );
					if ( !( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getEnabled() )
						g.setColor( Color.lightGray );
					g.drawString( ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getLabel(), ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getDrawPoint().x, ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getDrawPoint().y );

					g.setColor( Color.black );
					g.drawRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );
					g.fillRect( d.getItemBounds().getBounds().x + 2, d.getItemBounds().getBounds().y + d.getItemBounds().getBounds().height, d.getItemBounds().getBounds().width, 2 );
					g.fillRect( d.getItemBounds().getBounds().x + d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().y + 2, 2, d.getItemBounds().getBounds().height );

					// New menu means no previous selection.
					this.activeMenuItem = -1;
					activeMenuItemRect = new Rectangle( -1, -1, 0, 0 );

					eventHandled = true;
				}
			}
		}
		
		return eventHandled;
	}

	public boolean unhighlightMenu( MouseEvent e ) {
		boolean eventHandled = false;
		Graphics g = this.getGraphics();
		FontMetrics theFontMetrics = g.getFontMetrics();

		// Handle unhighlighting.
		if ( this.activeMenu != -1 ) {
			// Get menu object.
			int i = this.activeMenu;
			DesktopMenu d = ( DesktopMenu )dmb.getVector().elementAt( i );
			
			// Get menu item vector.
			Vector v = d.getVector();

			// Unhighlight current menu.
			// If Point is outside menu, redraw the menu.
			if ( !activeMenuRect.contains( e.getX(), e.getY() ) ) {
				if ( ( e.getX() < d.getItemBounds().getBounds().x ) || ( e.getX() > ( d.getItemBounds().getBounds().x + d.getItemBounds().width ) ) 
				|| ( e.getY() < d.getItemBounds().getBounds().y ) || ( e.getY() > d.getItemBounds().getBounds().y + d.getItemBounds().getBounds().height ) ) {
/*
				if ( ( e.getX() < d.getX() - Global.defaultMenuHSep ) || ( e.getX() > ( d.getX() + d.getItemBounds().width ) ) 
				|| ( e.getY() < this.getY() + this.getMenuBarHeight() + ( ( activeMenuItem ) * theFontMetrics.getHeight() ) ) || ( e.getY() > this.getY() + this.getMenuBarHeight() + ( ( activeMenuItem + 1 ) * theFontMetrics.getHeight() ) ) ) {
*/
					// Redraw menubar background behind this menu.
					g.setColor( dmb.getBackground() );
					g.fillRect( activeMenuRect.x, activeMenuRect.y, activeMenuRect.width, activeMenuRect.height );

					// Draw menu String or Image.
					g.setColor( Color.black );
					if ( d.getLabel() != null )
						g.drawString( d.getLabel(), d.getX(), d.getY() );
					else
						g.drawImage( ( ( DesktopImageMenu )d ).getImage(), d.getX(), d.getY(), this );
						
					// Redraw desktop background behind this menu.
					g.setColor( Color.lightGray );
					g.fillRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );

					g.drawRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );
					g.fillRect( d.getItemBounds().getBounds().x + 2, d.getItemBounds().getBounds().y + d.getItemBounds().getBounds().height, d.getItemBounds().getBounds().width, 2 );
					g.fillRect( d.getItemBounds().getBounds().x + d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().y + 2, 2, d.getItemBounds().getBounds().height );
					
					// New menu means no previous selection.
					this.activeMenu = -1;
					this.activeMenuItem = -1;
					activeMenuRect = new Rectangle( -1, -1, 0, 0 );
					activeMenuItemRect = new Rectangle( -1, -1, 0, 0 );

					eventHandled = true;
				}
			}
		}
		
		return eventHandled;
	}
	
	public boolean highlightMenuItem( MouseEvent e ) {
		boolean eventHandled = false;
		Graphics g = this.getGraphics();
		FontMetrics theFontMetrics = g.getFontMetrics();

		// Handle highlighting.
		if ( this.activeMenu != -1 ) {
			// Get menu object.
			int i = this.activeMenu;
			DesktopMenu d = ( DesktopMenu )dmb.getVector().elementAt( i );
			
			// Get menu item vector.
			Vector v = d.getVector();

			// Highlight current menu item.
			// If Point is inside menu item, redraw the menu item.
			// This is the current menu rect.  Replace this if().
			if ( ( e.getX() >= d.getX() - Global.defaultMenuHSep ) && ( e.getX() <= ( d.getX() + ( d.getLabel() == null ? ( ( DesktopImageMenu )d ).getImage().getWidth( this ) : theFontMetrics.stringWidth( d.getLabel() ) ) + Global.defaultMenuHSep ) ) 
			&& ( e.getY() >= this.getY() + this.getMenuBarHeight() ) && ( e.getY() <= this.getY() + this.getMenuBarHeight() +  ( v.size() * theFontMetrics.getHeight() ) ) ) {
				for ( int j = 0; j < v.size(); j++ ) {
					if ( !( ( DesktopMenuItem )v.elementAt( j ) ).getEnabled() )
						continue;
					Rectangle r = new Rectangle( ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().x, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().y, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().width, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().height );
					if ( r.getBounds().contains( e.getX(), e.getY() ) ) {
						if ( activeMenuItem != -1 && !activeMenuItemRect.contains( e.getX(), e.getY() ) ) {
							// Redraw menu background behind this menu item.
							g.setColor( dmb.getBackground() );
							g.fillRect( activeMenuItemRect.x, activeMenuItemRect.y, activeMenuItemRect.width, activeMenuItemRect.height );

							// Redraw menu item.
							g.setColor( Color.black );
							if ( !( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getEnabled() )
								g.setColor( Color.lightGray );
							g.drawString( ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getLabel(), ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getDrawPoint().x, ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getDrawPoint().y );
						}

						// Draw menu item background.
						g.setColor( Color.blue );
						activeMenuItemRect = new Rectangle( ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().x - Global.defaultMenuHSep + 1, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().y, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().width - 1, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().height );
//						activeMenuItemRect = new Rectangle( ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().x - Global.defaultMenuHSep, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().y, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().width, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().height );
						g.fillRect( activeMenuItemRect.x, activeMenuItemRect.y, activeMenuItemRect.width, activeMenuItemRect.height );

						// Draw menu item.
						g.setColor( dmb.getBackground() );
						g.drawString( ( ( DesktopMenuItem )v.elementAt( j ) ).getLabel(), ( ( DesktopMenuItem )v.elementAt( j ) ).getDrawPoint().x, ( ( DesktopMenuItem )v.elementAt( j ) ).getDrawPoint().y );

						// Set current menu.
						this.activeMenuItem = j;

						// Once found, we're done.
						eventHandled = true;
						break;
					}
				}
			}
		}
		
		return eventHandled;
	}
	
	public boolean highlightMenu( MouseEvent e ) {
		boolean eventHandled = false;
		Graphics g = this.getGraphics();
		FontMetrics theFontMetrics = g.getFontMetrics();

		// Handle menus.
		// Highlight current item.
		if ( this.activeMenu == -1 ) {
			for ( int i = 0; i < dmb.getVector().size(); i++ ) {
				// Get menu object.
				DesktopMenu d = ( DesktopMenu )dmb.getVector().elementAt( i );
				
				// Highlight current menu.
				// If Point is inside menu, redraw the menu and its items.
//			if ( activeMenuRect.contains( e.getX(), e.getY() ) {

				if ( ( e.getX() >= d.getX() - Global.defaultMenuHSep ) && ( e.getX() <= ( d.getX() + ( d.getLabel() == null ? ( ( DesktopImageMenu )d ).getImage().getWidth( this ) : theFontMetrics.stringWidth( d.getLabel() ) ) + Global.defaultMenuHSep ) ) 
				&& ( e.getY() >= this.getY() ) && ( e.getY() <= this.getMenuBarHeight() ) ) {

					// Draw menubar highlighting.
					g.setColor( Color.blue );
					activeMenuRect = new Rectangle( d.getX() - Global.defaultMenuHSep, this.getY(), ( ( d.getLabel() == null ? ( ( DesktopImageMenu )d ).getImage().getWidth( this ) : theFontMetrics.stringWidth( d.getLabel() ) ) ) + ( 2 * Global.defaultMenuHSep ), getMenuBarHeight() );
					g.fillRect( activeMenuRect.x, activeMenuRect.y, activeMenuRect.width, activeMenuRect.height );
					
					// Draw menu String or Image.
					g.setColor( Color.black );
					if ( d.getLabel() != null )
						g.drawString( d.getLabel(), d.getX(), d.getY() );
					else
						g.drawImage( ( ( DesktopImageMenu )d ).getImage(), d.getX(), d.getY(), this );

					// Get menu item vector.
					Vector v = d.getVector();
					// Draw menu background.
					g.setColor( dmb.getBackground() );
					g.fillRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );

					// Draw menu items.
					
					for ( int j = 0; j < v.size(); j++ ) {
						if ( ( ( DesktopMenuItem )v.elementAt( j ) ).getLabel().equals( Global.menuSep ) ) {
							g.setColor( Color.lightGray );
							g.fillRect( d.getItemBounds().getBounds().x, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().y + Global.defaultMenuItemVSep, d.getItemBounds().getBounds().width, 2 );
							continue;
						}
						
						g.setColor( Color.black );
						if ( !( ( DesktopMenuItem )v.elementAt( j ) ).getEnabled() )
							g.setColor( Color.lightGray );
						g.drawString( ( ( DesktopMenuItem )v.elementAt( j ) ).getLabel(), ( ( DesktopMenuItem )v.elementAt( j ) ).getDrawPoint().x, ( ( DesktopMenuItem )v.elementAt( j ) ).getDrawPoint().y );
					}
					
					g.setColor( Color.black );
					g.drawRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );
					g.fillRect( d.getItemBounds().getBounds().x + 2, d.getItemBounds().getBounds().y + d.getItemBounds().getBounds().height, d.getItemBounds().getBounds().width, 2 );
					g.fillRect( d.getItemBounds().getBounds().x + d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().y + 2, 2, d.getItemBounds().getBounds().height );

					// Set current menu.
					this.activeMenu = i;

					// Once found, we're done.
					eventHandled = true;
					break;
				}
			}
		}
		
		return eventHandled;
	}
	
	public void mouseDragged( MouseEvent e ) {
		// First, determine if the user has selected a different menu item.
			// Unhighlight current menu item.
		// Second, determine if the user has selected a different menu.
			// Unhighlight current menu.
		// Third, determine if the user has selected a new menu item.
			// Highlight new menu item.
		// Fourth, determine if the user has selected a new menu.
			// Highlight new menu.

		Graphics g = this.getGraphics();
		FontMetrics theFontMetrics = g.getFontMetrics();
		boolean eventHandled = false;

		if ( this.dragInProgress && ( this.dragItem != null ) ) {
			g.setColor( Color.lightGray );
			g.drawRect( this.dragItem.getBounds().x, this.dragItem.getBounds().y, this.imgFolder.getImage().getWidth( this ), this.imgFolder.getImage().getHeight( this ) );
			this.dragItem.setLocation( e.getX() - this.initialX, e.getY() - this.initialY );
			this.dragItem.setLabelLocation( this.dragItem.getX() + ( this.dragItem.getImage().getWidth( this ) / 2 ) - ( this.getGraphics().getFontMetrics().stringWidth( this.dragItem.getLabel() ) / 2 ), this.dragItem.getY() + ( this.dragItem.getImage().getHeight( this ) ) + ( this.getGraphics().getFontMetrics().getHeight() ) );
			g.setColor( Color.black );
			g.drawRect( this.dragItem.getBounds().x, this.dragItem.getBounds().y, this.imgFolder.getImage().getWidth( this ), this.imgFolder.getImage().getHeight( this ) );

			// Draw images.
			Enumeration enum = this.vectorImages.elements();
			int count = 0;
			
			while ( enum.hasMoreElements() ) {
				DesktopImage d = ( DesktopImage )enum.nextElement();
				if ( this.dragItem == d  )
					continue;
					
				Rectangle r = new Rectangle( d.getBounds().x, d.getBounds().y, d.getImage().getWidth( this ), d.getImage().getHeight( this ) );
				if ( r.contains( e.getX(), e.getY() ) ) {
					if ( this.highlightedImage != null && this.highlightedImage.getImage() != null && this.highlightedImage.getImage() != d.getImage() )
						g.drawImage( this.highlightedImage.getImage(), this.highlightedImage.getX(), this.highlightedImage.getY(), this );

					if ( count == 0 ) {
						if ( d.getImage() == this.imgTrash.getImage() )
							g.drawImage( this.imgTrashMask.getImage(), d.getX(), d.getY(), this );
						else
							g.drawImage( this.imgTrashFullMask.getImage(), d.getX(), d.getY(), this );
					}
					else if ( count == 1 ) {
						g.drawImage( this.imgDriveMask.getImage(), d.getX(), d.getY(), this );
					}
					
					this.highlightedImage = d;
					break;
				}
				
				count++;
			}
		}

		eventHandled = this.unhighlightMenuItem( e );
//System.out.println( "unhighlightMenuItem returned " + eventHandled );

//		if ( eventHandled )
		eventHandled = this.highlightMenuItem( e );
//System.out.println( "highlightMenuItem returned " + eventHandled );

//		if ( !eventHandled )
		eventHandled = this.unhighlightMenu( e );
//System.out.println( "unhighlightMenu returned " + eventHandled );

//		if ( eventHandled )
		eventHandled = this.highlightMenu( e );
//System.out.println( "highlightMenu returned " + eventHandled );

	}

	public void mouseMoved( MouseEvent e ) {
	}
		
	public void mouseClicked( MouseEvent e ) {
		if ( e.getClickCount() == 1 ) {
/*
			for ( int i = 0; i < vectorWindows.size(); i++ ) {
				DesktopFrame d = ( DesktopFrame )vectorWindows.elementAt( i );
				Rectangle r = d.getCloseBox();
				r.setLocation( r.getBounds().x + d.getBounds().x, r.getBounds().y + d.getBounds().y );
				if ( r.contains( e.getPoint() ) ) {
					d.setVisible( false );
//					d.repaint();
					this.vectorWindows.removeElementAt( i );
					Graphics g = this.getGraphics();
//	Unchanged.
					this.repaint();
					return;
				}
			}
*/
		}
		else if ( e.getClickCount() == 2 ) {

			for ( int i = 0; i < vectorImages.size(); i++ ) {
				DesktopImage d = ( DesktopImage )vectorImages.elementAt( i );
				Rectangle r = new Rectangle( d.getX(), d.getY(), d.getImage().getWidth( this ), d.getImage().getHeight( this ) );
				if ( r.contains( e.getPoint() ) ) {
					boolean found = false;
					
					for ( int count = 0; count < this.vectorWindows.size(); count ++ ) {
						DesktopFrame dwTemp = ( DesktopFrame )this.vectorWindows.elementAt( count );
						if ( dwTemp.getPath().equals( d.getPath() ) ) {
//System.out.println( "DEBUG: setting top frame from Desktop..." );
							this.setTopFrame( dwTemp );
							found = true;
							break;
						}
					}
					
					if ( found )
						break;
						
					Graphics g = this.getGraphics();

					DesktopFrame dw = new DesktopFrame( d.getPath() );
//					DesktopFrame dw = new DesktopFrame();
//					dw.setBounds( 100, 100, 200, 200 );
					dw.setLabel( d.getLabel() );
					dw.setPath( d.getPath() );
					dw.pack();
					dw.validate();
					dw.show();
					dw.toFront();
					dw.repaint();
				}
			}

		}
	}
		
	public void mouseEntered( MouseEvent e ) {
	}
		
	public void mouseExited( MouseEvent e ) {
	}
		
	public void mousePressed( MouseEvent e ) {
		// New click means no previous selection.
		this.activeMenu = -1;
		this.activeMenuItem = -1;

		Graphics g = this.getGraphics();
		FontMetrics theFontMetrics = g.getFontMetrics();

		// Draw images.
		Enumeration enum = this.vectorImages.elements();

		while ( enum.hasMoreElements() ) {
			DesktopImage d = ( DesktopImage )enum.nextElement();
			Rectangle r = new Rectangle( d.getBounds().x, d.getBounds().y, d.getImage().getWidth( this ), d.getImage().getHeight( this ) );
			if ( r.contains( e.getX(), e.getY() ) ) {
				this.initialX = e.getX() - r.getBounds().x;
				this.initialY = e.getY() - r.getBounds().y;
				this.dragInProgress = true;
				this.dragItem = d;
				return;
			}
		}			

		if ( dmb.contains( e.getX(), e.getY() ) ) {
			// Handle menu selection.
			for ( int i = 0; i < dmb.getVector().size(); i++ ) {
				// Get menu object.
				DesktopMenu d = ( DesktopMenu )dmb.getVector().elementAt( i );

				// Determine if we're inside this menu.
				if ( ( e.getX() >= d.getX() - Global.defaultMenuHSep ) && ( e.getX() <= ( d.getX() + ( d.getLabel() == null ? ( ( DesktopImageMenu )d ).getImage().getWidth( this ) : theFontMetrics.stringWidth( d.getLabel() ) ) + Global.defaultMenuHSep ) ) 
				&& ( e.getY() >= this.getY() ) && ( e.getY() <= this.getMenuBarHeight() ) ) {
					// Draw menubar highlighting.
					g.setColor( Color.blue );
	//				g.fillRect( d.getX() - Global.defaultMenuHSep, this.getY(), ( d.getLabel() == null ? d.getImage().getWidth( this ) : theFontMetrics.stringWidth( d.getLabel() ) ) + ( 2 * Global.defaultMenuHSep ), getMenuBarHeight() );
					activeMenuRect = new Rectangle( d.getX() - Global.defaultMenuHSep, this.getY(), ( d.getLabel() == null ? ( ( DesktopImageMenu )d ).getImage().getWidth( this ) : theFontMetrics.stringWidth( d.getLabel() ) ) + ( 2 * Global.defaultMenuHSep ), getMenuBarHeight() );
					g.fillRect(	activeMenuRect.x, activeMenuRect.y, activeMenuRect.width, activeMenuRect.height );

					// Draw menu String or Image.
					g.setColor( Color.black );
					if ( d.getLabel() != null )
						g.drawString( d.getLabel(), d.getX(), d.getY() );
					else
						g.drawImage( ( ( DesktopImageMenu )d ).getImage(), d.getX(), d.getY(), this );
					
					// Get menu item vector.
					Vector v = d.getVector();

					DesktopMenuItem dmi = ( DesktopMenuItem )v.elementAt( 0 );
					if ( dmi.getLabel().equals( Global.menuItemEmptyTrash ) ) {
						DesktopImage di = ( ( DesktopImage )this.vectorImages.elementAt( 1 ) );
						String path = di.getPath();
						File f = new File( path, Global.trashDisplayString );
						if ( f.exists() ) {
							String array[] = f.list();
							
							if ( array == null || array.length == 0 )
								dmi.setEnabled( false );
							else
								dmi.setEnabled( true );
						}
					}
					
					// Draw menu background.
					g.setColor( dmb.getBackground() );
					g.fillRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );

					// Draw menu items.
					for ( int j = 0; j < v.size(); j++ ) {
						if ( ( ( DesktopMenuItem )v.elementAt( j ) ).getLabel().equals( Global.menuSep ) ) {
							g.setColor( Color.lightGray );
							g.fillRect( d.getItemBounds().getBounds().x, ( ( DesktopMenuItem )v.elementAt( j ) ).getBounds().y + Global.defaultMenuItemVSep, d.getItemBounds().getBounds().width, 2 );
							continue;
						}
						
						g.setColor( Color.black );
						if ( !( ( DesktopMenuItem )v.elementAt( j ) ).getEnabled() )
							g.setColor( Color.lightGray );
						g.drawString( ( ( DesktopMenuItem )v.elementAt( j ) ).getLabel(), ( ( DesktopMenuItem )v.elementAt( j ) ).getDrawPoint().x, ( ( DesktopMenuItem )v.elementAt( j ) ).getDrawPoint().y );
					}

					g.setColor( Color.black );
					g.drawRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );
					g.fillRect( d.getItemBounds().getBounds().x + 2, d.getItemBounds().getBounds().y + d.getItemBounds().getBounds().height, d.getItemBounds().getBounds().width, 2 );
					g.fillRect( d.getItemBounds().getBounds().x + d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().y + 2, 2, d.getItemBounds().getBounds().height );
					
					// Set current menu.
					this.activeMenu = i;

					// Once found, we're done.
					break;
				}
			}
		}
	}
		
	public void mouseReleased( MouseEvent e ) {
		Graphics g = this.getGraphics();
		FontMetrics theFontMetrics = g.getFontMetrics();

		// Handle redraw of current menu.
		if ( this.activeMenu != -1 ) {
			// Get menu object.
			int i = this.activeMenu;
			DesktopMenu d = ( DesktopMenu )dmb.getVector().elementAt( i );

			doFlashMenuItem( d );
			
			// Get menu item vector.
			Vector v = d.getVector();

			// Redraw desktop background behind this menu.
			g.setColor( Color.lightGray );
//			g.fillRect( d.getX() - Global.defaultMenuHSep, this.getY() + this.getMenuBarHeight(), tempWidth + ( 2 * Global.defaultMenuHSep ), v.size() * theFontMetrics.getHeight() );
			g.fillRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );

			g.drawRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );
			g.fillRect( d.getItemBounds().getBounds().x + 2, d.getItemBounds().getBounds().y + d.getItemBounds().getBounds().height, d.getItemBounds().getBounds().width, 2 );
			g.fillRect( d.getItemBounds().getBounds().x + d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().y + 2, 2, d.getItemBounds().getBounds().height );

			// Redraw menubar background behind this menu.
			g.setColor( dmb.getBackground() );
			g.fillRect( d.getX() - Global.defaultMenuHSep, this.getY(), ( d.getLabel() == null ? ( ( DesktopImageMenu )d ).getImage().getWidth( this ) : theFontMetrics.stringWidth( d.getLabel() ) ) + ( 2 * Global.defaultMenuHSep ), getMenuBarHeight() );
			g.setColor( Color.black );
			
			// Draw menu String or Image.
			if ( d.getLabel() != null )
				g.drawString( d.getLabel(), d.getX(), d.getY() );
			else
				g.drawImage( ( ( DesktopImageMenu )d ).getImage(), d.getX(), d.getY(), this );
				
//			String s = ( ( DesktopMenuItem )d.getVector().elementAt( this.activeMenuItem ) ).getLabel();
			this.handleMenuItem();

			// No active menu.
			this.activeMenu = -1;
			this.activeMenuItem = -1;
			activeMenuRect = new Rectangle( -1, -1, 0, 0 );
			activeMenuItemRect = new Rectangle( -1, -1, 0, 0 );
		}
		else if ( this.dragItem != null && this.dragInProgress ) {
			if ( this.highlightedImage != null && this.highlightedImage.getImage() != null ) {
				Rectangle r;
				if ( this.highlightedImage.getImage() != null )
					r = new Rectangle( this.highlightedImage.getBounds().x, this.highlightedImage.getBounds().y, this.highlightedImage.getImage().getWidth( this ), this.highlightedImage.getImage().getHeight( this ) );
				else
					r = new Rectangle( this.highlightedImage.getBounds().x, this.highlightedImage.getBounds().y, this.highlightedImage.getBounds().width, this.highlightedImage.getBounds().height );

				if ( r.contains( e.getX(), e.getY() ) ) {
					// Move folder
					String path = this.highlightedImage.getPath();
					if ( path != null ) {
						File f = new File( path );
						if ( f.exists() && f.isDirectory() ) {
							if ( this.dragItem != this.vectorImages.elementAt( 1 ) ) {
	//							System.out.println( "Oops!" );
								File fSource = new File( this.dragItem.getPath() );
								boolean b = this.moveItem( fSource, f );
								this.vectorImages.removeElement( this.dragItem );
				
								for ( int j = 0; j < vectorWindows.size(); j++ ) {
									DesktopFrame d = ( DesktopFrame )vectorWindows.elementAt( j );
									if ( d.getPath() == this.dragItem.getPath() ) {
										d.setVisible( false );
										this.vectorWindows.removeElementAt( j );
									}
								}
								
								if ( this.highlightedImage.getImage() == ( ( DesktopImage )this.vectorImages.elementAt( 0 ) ).getImage() ) {
//								if ( this.highlightedImage.getImage() == this.imgTrash.getImage() || this.highlightedImage.getImage() == this.imgTrashFull.getImage() ) {
									r = ( ( DesktopImage )this.vectorImages.elementAt( 0 ) ).getBounds();
									this.vectorImages.setElementAt( this.imgTrashFull, 0 );
									( ( DesktopImage )this.vectorImages.elementAt( 0 ) ).setBounds( r );
									this.repaint();
								}
							}
							else
								System.out.println( "Cannot move root volume." );
						}
					}		
				}
			}

			this.dragItem.setLocation( e.getX() - this.initialX, e.getY() - this.initialY );
			this.dragItem.setLabelLocation( this.dragItem.getX() + ( this.dragItem.getImage().getWidth( this ) / 2 ) - ( this.getGraphics().getFontMetrics().stringWidth( this.dragItem.getLabel() ) / 2 ), this.dragItem.getY() + ( this.dragItem.getImage().getHeight( this ) ) + ( this.getGraphics().getFontMetrics().getHeight() ) );
			this.initialX = 0;
			this.initialY = 0;
			this.dragInProgress = false;
			this.dragItem = null;
			this.highlightedImage = null;

//	Unchanged.
			this.repaint();
		}
	}
	
//	public boolean handleMenuItem( String s ) {
	public boolean handleMenuItem() {
		boolean returnValue = false;

		// Handle active menu item.
		if ( ( this.activeMenu != -1 ) && ( this.activeMenuItem != -1 ) ) {
			// Get menu object.
			DesktopMenu dm = ( DesktopMenu )dmb.getVector().elementAt( this.activeMenu );
			
			// Get menu item.
			String s = ( ( DesktopMenuItem )dm.getVector().elementAt( this.activeMenuItem ) ).getLabel();

			// Handle shutdown menu item.
			if ( s.equals( Global.menuItemShutdown ) ) {
				this.saveState();
				System.exit( 0 );
				returnValue = true;
			}
			// Handle About... menu item.
			else if ( s.equals( Global.menuItemNew ) ) {
				boolean done = false;
				int i = -1;
				
				DesktopImage di = ( ( DesktopImage )this.vectorImages.elementAt( 1 ) );
				String path = di.getPath();
				StringBuffer sb = new StringBuffer();
				File f = new File( path, sb.toString() );

				while ( !done ) {
					sb.append( Global.newItemName );

					if ( i >= 0 )
						sb.append( Global.newItemCopyName );

					if ( i > 0 )
						sb.append( " " + i );

					i++;

					f = new File( path, sb.toString() );
		
					if ( !f.exists() ) {
						boolean b = f.mkdir();
						done = true;
						break;
					}

					sb = new StringBuffer();
				}

				DesktopImage d = new DesktopImage();
				d.setLabel( sb.toString() );
				d.setPath( f.getPath() );
				d.setImage( imgFolder.getImage() );
				d.setLocation( di.getBounds().x, di.getBounds().y + ( di.getImage().getHeight( this ) * 3 ) );
				d.setLabelLocation( d.getX() + ( d.getImage().getWidth( this ) / 2 ) - ( this.getGraphics().getFontMetrics().stringWidth( d.getLabel() ) / 2 ), d.getY() + ( d.getImage().getHeight( this ) ) + ( this.getGraphics().getFontMetrics().getHeight() ) );
				d.addMouseListener( this );
				this.vectorImages.addElement( d );
				this.repaint();
			}
			else if ( s.equals( Global.menuItemEmptyTrash ) ) {
				DesktopImage di = ( ( DesktopImage )this.vectorImages.elementAt( 1 ) );
				String path = di.getPath();
				File f = new File( path, Global.trashDisplayString );
				if ( f.exists() && f.isDirectory() ) {
					boolean b = this.emptyDir( f );
					if ( b ) {
						Rectangle r = ( ( DesktopImage )this.vectorImages.elementAt( 0 ) ).getBounds();
						this.vectorImages.setElementAt( this.imgTrash, 0 );
						( ( DesktopImage )this.vectorImages.elementAt( 0 ) ).setBounds( r );
						this.repaint();
					}
				}
			}
			// Handle About... menu item.
			else if ( s.equals( Global.menuItemAboutMac ) ) {
//				DesktopWindow d = new DesktopWindow( new Frame() );
				DesktopDialog d = new DesktopDialog();
				d.setX( 100 );
				d.setY( 100 );
				d.setWidth( 200 );
				d.setHeight( 100 );
				String s1 = new String( System.getProperty( "os.name" ) + " " + System.getProperty( "os.arch" ) + " " + System.getProperty( "os.version" ) );
				d.setLabel( s1 );
				//System.getProperty( "os.name" ) + " " + System.getProperty( "os.arch" ) + " " + System.getProperty( "os.version" ) );
				d.show();
				d.toFront();
				d.paint( this.getGraphics() );

				this.doDelay();
				this.repaint();
				returnValue = true;
			}
		}
		
		return returnValue;
	}
	
	public boolean moveItem( File f, File dir ) {
		// Recursive method to move a directory's contents,
		// then delete the directory.
		boolean retVal = false;
		
		String name = f.getName();
		StringBuffer sb = new StringBuffer( name );
		String array[] = dir.list();
		boolean done = false, found = false;
		int i = 0;			
			
		if ( array != null ) {
			while ( !done ) {
				found = false;
				
				for ( int count = 0; count < array.length; count ++ ) {
					String temp = array[ count ];
					if ( temp.equals( sb.toString() ) ) {
						found = true;
						break;
					}
				}

				if ( found ) {
					if ( i >= 0 )
						sb.append( Global.newItemCopyName );

					if ( i > 0 )
						sb.append( " " + i );

					i++;
				}
				else
					done = true;
			}
		}
			
		File fNew = new File( dir.getPath(), sb.toString() );

		if ( f.isFile() ) {
			try {
				FileOutputStream fos = new FileOutputStream( fNew );
				DataOutputStream dos = new DataOutputStream( fos );

				FileInputStream fis = new FileInputStream( f );
				DataInputStream dis = new DataInputStream( fis );
				
				int offset = 0;
				int size = 64;
				byte[] b = new byte[ size ];
				
				while ( dis.available() > 0 ) {
					if ( dis.available() < size ) {
						size = dis.available();
						b = new byte[ size ];
					}
					
					int amount = dis.read( b );
					if ( amount <= 0 )
						break;

					dos.write( b );
					
					offset += amount;
				}
			
				dos.flush();
				
				dos.close();
				fos.close();
				
				dis.close();
				fis.close();
				
				f.delete();
				
				retVal = true;
			}
			catch ( IOException e ) {
				System.out.println( "IOException moving file..." );
			}
		}
		else if ( f.isDirectory() ) {
			array = f.list();
			retVal = fNew.mkdirs();
	
			if ( array != null /*&& array.length > 0*/ ) {
				for ( int count = 0; count < array.length; count ++ ) {
					String temp = array[ count ];
					File f2 = new File( f, temp );
					
//					if ( f2.isFile() )
						retVal = this.moveItem( f2, fNew );
//					else if ( f2.isDirectory() )
//						retVal = fNew.mkdirs();
				}
			}
			
			if ( retVal ) {
				retVal = this.emptyDir( f, true );
				retVal = f.delete();
			}
		}

		return retVal;
	}
	
	public boolean emptyDir( File dir ) {
		// Recursive method to remove a directory's contents,
		// then delete the directory.
		boolean b = false;
		
		String array[] = dir.list();
			
		if ( array != null ) {
			for ( int count = 0; count < array.length; count ++ ) {
				String temp = array[ count ];
				File f1 = new File( dir, temp );
				
				if ( f1.isFile() ) {
					b = f1.delete();
				}
				else if ( f1.isDirectory() ) {
					b = this.emptyDir( f1 );
					b = f1.delete();
//if ( !b ) System.out.println( "Cannot delete " + f1.toString() );
				}
			}
		}

		return b;
	}
	
	public boolean emptyDir( File dir, boolean remove ) {
		// Recursive method to remove a directory's contents,
		// then delete the directory.
		boolean b = false;
		
		String array[] = dir.list();
			
		if ( array != null ) {
			for ( int count = 0; count < array.length; count ++ ) {
				String temp = array[ count ];
				File f1 = new File( dir, temp );
				
				if ( f1.isFile() )
					b = f1.delete();
				else if ( f1.isDirectory() ) {
					b = this.emptyDir( f1, remove );
					
					if ( remove )
						f1.delete();
					
//if ( !b ) System.out.println( "Cannot delete " + f1.toString() );
				}
			}
		}

		return b;
	}
	
	public void redrawOpenFrames() {
		// Draw windows.
		Enumeration e = this.vectorWindows.elements();

		while ( e.hasMoreElements() ) {
			DesktopFrame d = ( DesktopFrame )e.nextElement();
			
			d.repaint();
		}
	}
	
	public void paint( Graphics g2 ) {
//	public void paint( Graphics g ) {
		FontMetrics theFontMetrics = g.getFontMetrics();

		// Draw splash image.
		if ( this.bSplash ) {
			g.drawImage( imgSplash.getImage(), this.getX() + ( ( this.getWidth() / 2 ) - ( imgSplash.getImage().getWidth( this ) / 2 ) ), this.getY() + ( ( this.getHeight() / 2 ) - ( imgSplash.getImage().getHeight( this ) / 2 ) ), this );
			g.setColor( Color.black );
			g.drawString( Global.splashDisplayString, this.getX() + ( this.getWidth() / 2 ) - ( theFontMetrics.stringWidth( Global.splashDisplayString ) / 2 ), this.getY() + ( ( this.getHeight() / 2 ) + ( imgSplash.getImage().getHeight( this ) / 2 ) ) - ( 3 * theFontMetrics.getHeight() ) );
			g.drawString( Global.startupDisplayString, this.getX() + ( this.getWidth() / 2 ) - ( theFontMetrics.stringWidth( Global.startupDisplayString ) / 2 ), this.getY() + ( ( this.getHeight() / 2 ) + ( imgSplash.getImage().getHeight( this ) / 2 ) ) - ( 2 * theFontMetrics.getHeight() ) );
//			this.bSplash = false;

g2.drawImage( offscreenImage, 0, 0, this );

			return;
		}

		// Draw menubar.
		g.setColor( dmb.getBackground() );
		g.fillRect( dmb.getX(), dmb.getY(), dmb.getWidth(), dmb.getHeight() );

//		dmb.repaint();
		
		// Draw desktop background.
		g.setColor( Color.lightGray );
		g.fillRect( this.getX(), this.getMenuBarHeight(), this.getWidth(), this.getHeight() - this.getMenuBarHeight() );

		g.setColor( Color.black );

		// Draw images.
		Enumeration e = this.vectorImages.elements();

		g.setFont( this.iconFont );
		theFontMetrics = g.getFontMetrics();

		while ( e.hasMoreElements() ) {
			DesktopImage d = ( DesktopImage )e.nextElement();
			g.drawImage( d.getImage(), d.getX(), d.getY(), this );
			g.setColor( Color.black );
			if( d.getImage() == this.imgTrash.getImage() || d.getImage() == this.imgTrashFull.getImage() ) {
				g.setColor( Color.white );
				g.fillRect( d.getX() - 1, d.getY() + ( d.getImage().getHeight( this ) ) + 1, theFontMetrics.stringWidth( d.getLabel() ) + 2, theFontMetrics.getHeight() - theFontMetrics.getLeading() + 2 );
				g.setColor( Color.black );
				g.drawString( d.getLabel(), d.getX(), d.getY() + ( d.getImage().getHeight( this ) ) + ( theFontMetrics.getAscent() ) + 1 );
			}
			else {
				g.setColor( Color.white );
				g.fillRect( d.getX() + ( d.getImage().getWidth( this ) / 2 ) - ( theFontMetrics.stringWidth( d.getLabel() ) / 2 ) - 1, d.getY() + ( d.getImage().getHeight( this ) ) + 1, theFontMetrics.stringWidth( d.getLabel() ) + 2, theFontMetrics.getHeight() - theFontMetrics.getLeading() + 2 );
				g.setColor( Color.black );
				g.drawString( d.getLabel(), d.getX() + ( d.getImage().getWidth( this ) / 2 ) - ( theFontMetrics.stringWidth( d.getLabel() ) / 2 ), d.getY() + ( d.getImage().getHeight( this ) ) + ( theFontMetrics.getAscent() ) + 1 );
			}
		}

		g.setFont( this.menuFont );
		theFontMetrics = g.getFontMetrics();

		// Draw menus.
		e = this.dmb.getVector().elements();

		while ( e.hasMoreElements() ) {
			DesktopMenu d = ( DesktopMenu )e.nextElement();

			if ( d.getLabel() != null )
				g.drawString( d.getLabel(), d.getX(), d.getY() );
			else
				g.drawImage( ( ( DesktopImageMenu )d ).getImage(), d.getX(), d.getY(), this );
		}
		
//if ( this.activeMenu == -1 ) {
//return;	

		// Draw windows.
		e = this.vectorWindows.elements();

		while ( e.hasMoreElements() ) {
			DesktopFrame d = ( DesktopFrame )e.nextElement();
			d.setVisible( true );

			d.repaint();
		}
//}

g2.drawImage( offscreenImage, 0, 0, this );
	}
	
	public static boolean windowExists( String s ) {
//System.out.println( "DEBUG: Incoming path = " + s );
		boolean found = false;
		int i = 0;
		
		while ( i < vectorWindows.size() ) {
			DesktopFrame d = ( DesktopFrame )vectorWindows.elementAt( i );
//System.out.println( "DEBUG: d.getPath() = " + d.getPath() );
			if ( d.getPath().equalsIgnoreCase( s ) ) {
				Desktop.setTopFrame( d );
//				d.toFront();
//				d.repaint();
				found = true;
				break;
			}
			
			i++;
		}

		return found;
	}
	
	public static void setTopFrame( DesktopFrame d ) {
		boolean found = false;
		int i = 0;
		
		while ( i < vectorWindows.size() ) {
			if ( vectorWindows.elementAt( i ) == d ) {
				d.toFront();
				d.repaint();
				found = true;
				break;
			}
			
			i++;
		}

		if ( !found )
			return;
			
		DesktopFrame temp = ( DesktopFrame )vectorWindows.elementAt( i );
		
		while ( i < vectorWindows.size() - 1 ) {
			vectorWindows.setElementAt( vectorWindows.elementAt( i + 1 ), i );
			i++;
		}

		vectorWindows.setElementAt( temp, vectorWindows.size() - 1 );
	}

	public static void setTopFrame( String s ) {
		boolean found = false;
		int i = 0;
		
		while ( i < vectorWindows.size() ) {
			DesktopFrame d = ( DesktopFrame )vectorWindows.elementAt( i );
			if ( d.getPath().equals( s ) ) {
				d.toFront();
				d.repaint();
				found = true;
				break;
			}
			
			i++;
		}

		if ( !found )
			return;
			
		DesktopFrame temp = ( DesktopFrame )vectorWindows.elementAt( i );
		
		while ( i < vectorWindows.size() - 1 ) {
			vectorWindows.setElementAt( vectorWindows.elementAt( i + 1 ), i );
			i++;
		}

		vectorWindows.setElementAt( temp, vectorWindows.size() - 1 );
	}

	public static void adjustClipRect( Rectangle r ) {
		tempClipRect.setBounds( r.getBounds().x, r.getBounds().y, r.getBounds().width, r.getBounds().height );
//		this.getGraphics().clipRect( r.getBounds().x, r.getBounds().y, r.getBounds().width, r.getBounds().height );
	}

	public void update( Graphics g ) {
//		g.clipRect( tempClipRect.getBounds().x, tempClipRect.getBounds().y, tempClipRect.getBounds().width, tempClipRect.getBounds().height );
		paint( g );
	}

/*
	public DesktopWindow getActiveWindow() {
		return this.activeWindow;
	}

	public void setActiveWindow( DesktopWindow d ) {
		this.activeWindow = d;
	}
*/
	public void setX( int i ) {
		this.setLocation( i, this.getLocation().y );
	}

	public int getX() {
		return this.getLocation().x;
	}

	public void setY( int i ) {
		this.setLocation( this.getLocation().x, i );
	}

	public int getY() {
		return this.getLocation().y;
	}

	public void setWidth( int i ) {
		this.setSize( i, this.getSize().height );
	}

	public int getWidth() {
		return this.getSize().width;
	}

	public void setHeight( int i ) {
		this.setSize( this.getSize().width, i );
	}

	public int getHeight() {
		return this.getSize().height;
	}

	public void setMenuBarHeight( int i) {
		this.menuBarHeight = i;
	}

	public int getMenuBarHeight() {
		return this.menuBarHeight;
	}

	public static Image getImgDoc() {
		return ( ( Image )vectorUtil.elementAt( 0 ) );
	}

	public static Image getImgFolder() {
		return ( ( Image )vectorUtil.elementAt( 1 ) );
	}

	public int drawIcon( Image theImage, int x, int y ) {
		this.getGraphics().drawImage( theImage, x, y, this );
		return( theImage.getWidth( this ) );
	}
/*	
	public static void addIcon( DesktopFrameItem d ) {
		boolean found = false;
		
		for ( int i = 0; i < vectorIcons.size(); i++ ) {
			if ( vectorIcons.elementAt( i ) == d ) {
				found = true;
			}
		}
		
		if ( !found ) {
			vectorIcons.addElement( d );
		}
	}
*/
	public static void addWindow( DesktopFrame d ) {
		boolean found = false;
		
		for ( int i = 0; i < vectorWindows.size(); i++ ) {
			if ( vectorWindows.elementAt( i ) == d ) {
				( ( Window )vectorWindows.elementAt( i ) ).toFront();
				Desktop.setTopFrame( d );
				found = true;
			}
		}
		
		if ( !found ) {
			vectorWindows.addElement( d );
		}
	}
	
	public static void removeWindow( DesktopFrame d ) {
		for ( int i = 0; i < vectorWindows.size(); i++ ) {
			if ( vectorWindows.elementAt( i ) == d ) {
				vectorWindows.removeElementAt( i );
			}
		}
	}
	
	public void doDelayShort() {
		int delay = 0;
		Timer t1 = new Timer();
		t1.addListener( this );
		while ( delay < 1 );
		t1.removeListener( this );
	}
	
	public void doDelay() {
		this.delay = 0;
		Timer t1 = new Timer();
		t1.addListener( this );
		while ( delay < Global.splashDelay ) {
			;
		}
		
		this.delay = 0;
		t1.removeListener( this );
	}
	
	public void doDelay( int i ) {
		// Note: i will cause a delay of ( i <seconds> * Global.splashDelay ).
		this.delay = 0;
		Timer t1 = new Timer( i );
		t1.addListener( this );
		while ( delay < Global.splashDelay );
		this.delay = 0;
		t1.removeListener( this );
	}
	
	public void doMenuDelay() {
		// Note: i will cause a delay of ( i <seconds> * Global.splashDelay ).

		Calendar c = Calendar.getInstance();
		Calendar old = Calendar.getInstance();
		
		old.add( Calendar.MILLISECOND, Global.menuDelayMillis );
		
		while ( !c.after( old ) ) {
			c = Calendar.getInstance();
		}
	}
	
	public void timerExpired( TimerEvent evt ) {
		this.delay++;
	}

	public boolean checkTrashExists() {
		DesktopImage di = ( ( DesktopImage )this.vectorImages.elementAt( 1 ) );
		String path = di.getPath();
		String s = new String( Global.trashDisplayString );
		File f = new File( path, s );
		boolean retVal = false;
		
		if ( f.exists() )
			retVal = true;
			
		if ( retVal ) {
			di = ( ( DesktopImage )this.vectorImages.elementAt( 0 ) );
			di.setPath( f.toString() );
		}
		
		return retVal;
	}
	
	public boolean createTrashDir() {
		DesktopImage di = ( ( DesktopImage )this.vectorImages.elementAt( 1 ) );
		String path = di.getPath();
		String s = new String( Global.trashDisplayString );
		File f = new File( path, s );
		boolean retVal = false;
			
		retVal = f.mkdir();

		if ( retVal ) {
			di = ( ( DesktopImage )this.vectorImages.elementAt( 0 ) );
			di.setPath( f.toString() );
		}
		
		return retVal;
	}
	
	public void doFlashMenuItem( DesktopMenu d ) {
		Graphics g = this.getGraphics();
		
		if ( ( this.activeMenu != -1 ) && ( this.activeMenuItem != -1 ) ) {
			// Get menu item vector.
			Vector v = d.getVector();

			// Unhighlight current menu item.
			// If Point is outside menu item, redraw the menu item.
			for ( int i = 0; i < 3; i++ ) {
				// Redraw menu background behind this menu item.
				g.setColor( dmb.getBackground() );
				g.fillRect( activeMenuItemRect.x, activeMenuItemRect.y, activeMenuItemRect.width, activeMenuItemRect.height );

				g.setColor( Color.black );
				g.drawRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );
//				g.fillRect( d.getItemBounds().getBounds().x + 2, d.getItemBounds().getBounds().y + d.getItemBounds().getBounds().height, d.getItemBounds().getBounds().width, 2 );
//				g.fillRect( d.getItemBounds().getBounds().x + d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().y + 2, 2, d.getItemBounds().getBounds().height );

				// Redraw menu item.
				g.setColor( Color.black );
				g.drawString( ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getLabel(), ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getDrawPoint().x, ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getDrawPoint().y );

				this.doMenuDelay();

				// Redraw menu background behind this menu item.
				g.setColor( Color.blue );
				g.fillRect( activeMenuItemRect.x, activeMenuItemRect.y, activeMenuItemRect.width, activeMenuItemRect.height );

				g.setColor( Color.black );
				g.drawRect( d.getItemBounds().getBounds().x, d.getItemBounds().getBounds().y, d.getItemBounds().getBounds().width, d.getItemBounds().getBounds().height );

				// Redraw menu item.
				g.setColor( Color.white );
				g.drawString( ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getLabel(), ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getDrawPoint().x, ( ( DesktopMenuItem )v.elementAt( activeMenuItem ) ).getDrawPoint().y );

				this.doMenuDelay();
			}
		}
	}

	private void saveState() {
		DesktopImage di = ( ( DesktopImage )this.vectorImages.elementAt( 1 ) );
		String s = new String( di.getPath() + "Desktop.ser" );
		try {
			FileOutputStream fos = new FileOutputStream( s );
			ObjectOutputStream outStream = new ObjectOutputStream( fos );
			Vector v = new Vector();
			v = ( Vector )this.vectorWindows.clone();

			Vector temp = new Vector();
			
			for ( int i = 0; i < v.size(); i++ ) {
				DesktopFrame df = ( DesktopFrame )v.elementAt( i );
				temp.addElement( df.getPath() );
				temp.addElement( df.getLabel() );
				
				if ( df.getShade() ) {
					df.restoreHeight();
				}
				
				temp.addElement( df.getBounds() );
			}

			outStream.writeObject( temp );

			temp = new Vector();

			v = new Vector();
			v = ( Vector )this.vectorImages.clone();

			for ( int i = 2; i < v.size(); i++ ) {
				DesktopImage d = ( DesktopImage )v.elementAt( i );
				temp.addElement( d.getPath() );
				temp.addElement( d.getLabel() );
				temp.addElement( d.getBounds() );
			}

			outStream.writeObject( temp );

			outStream.flush();
			outStream.close();
		}
		catch ( IOException e ) {
			System.out.println( "Couldn't save state." );
			System.out.println( "s = " + s );
			e.printStackTrace();
		}
	}

	private void restoreState() {
		DesktopImage di = ( ( DesktopImage )this.vectorImages.elementAt( 1 ) );
		String s = new String( "\\Desktop.ser" );
//		String s = new String( di.getPath() + "Desktop.ser" );
		try {
			FileInputStream fis = new FileInputStream( s );
			ObjectInputStream inStream = new ObjectInputStream( fis );
			Vector v = new Vector();
			v = ( Vector )inStream.readObject();
			
			Rectangle r = new Rectangle();
			
			for ( int i = 0; i < v.size(); i++ ) {
				s = ( String )v.elementAt( i );
				
				DesktopFrame df = new DesktopFrame( s );
	
				i++;
				s = ( String )v.elementAt( i );
				df.setLabel( s );

				i++;
				r = ( Rectangle )v.elementAt( i );
				df.setBounds( r );
				
				df.pack();
				df.validate();
				df.show();
				df.toFront();
				df.repaint();
			}

			v = new Vector();
			v = ( Vector )inStream.readObject();

			for ( int i = 0; i < v.size(); i++ ) {
				DesktopImage d = new DesktopImage();
				
				s = ( String )v.elementAt( i );
				d.setPath( s );
					
				i++;
				s = ( String )v.elementAt( i );
				d.setLabel( s );
				
				i++;
				r = ( Rectangle )v.elementAt( i );
				d.setBounds( r );
				
				d.setImage( imgFolder.getImage() );
				d.setLocation( di.getBounds().x, di.getBounds().y + ( di.getImage().getHeight( this ) * 3 ) );
				d.setLabelLocation( d.getX() + ( d.getImage().getWidth( this ) / 2 ) - ( this.getGraphics().getFontMetrics().stringWidth( d.getLabel() ) / 2 ), d.getY() + ( d.getImage().getHeight( this ) ) + ( this.getGraphics().getFontMetrics().getHeight() ) );
				d.addMouseListener( this );
				this.vectorImages.addElement( d );
			}

			inStream.close();
		}
		catch ( ClassNotFoundException e ) {
			System.out.println( "ClassNotFoundException in retrieveState()." );
		}
		catch ( IOException e ) {
			System.out.println( "Couldn't retrieve state." );
			System.out.println( "s = " + s );
			e.printStackTrace();
		}
	}
}

